#include "typedef.h"
#include "pwm_ctrl.h"
#include "led_ctrl.h"
#include "portg_ctrl.h"
#include "adc_ctrl.h"
#include "audio.h"
#include <stdlib.h>

typedef struct stereo {
	s2 l;
	s2 r;
} t_stereo;

static s4 decode_get(t_stereo *decode_data);
#ifdef LEVEL
static u1 level2bar(s4 data);
#endif /* LEVEL */

static t_stereo ringbuf[1152 + 1];
static u4 decode_putp = 0UL, decode_getp = 0UL;

#define RINGBUF_SIZE_MAX (sizeof(ringbuf)/sizeof(t_stereo))
#define p_inc(p, max) (((p + 1UL) >= max) ? 0UL : (p + 1UL))

/* Oobt@ɃfR[hf[^ */
s4 decode_put(s2 left, s2 right)
{
	u4 putp = p_inc(decode_putp, RINGBUF_SIZE_MAX);
	s4 r = PUT_NG;

	if(putp != decode_getp)
	{
		ringbuf[decode_putp].l = left;
		ringbuf[decode_putp].r = right;

		decode_putp = putp;
		r = PUT_OK;
	}
	return r;
}

/* Oobt@fR[hf[^ */
static s4 decode_get(t_stereo *decode_data)
{
	s4 r = GET_NG;

	if(decode_putp != decode_getp)
	{
		decode_data->l = ringbuf[decode_getp].l;
		decode_data->r = ringbuf[decode_getp].r;

		decode_getp = p_inc(decode_getp, RINGBUF_SIZE_MAX);
		r = GET_OK;
	}
	return r;
}

/* Oobt@ */
s4 decode_empty(void)
{
	s4 r = EMPTY_NG;

	if(decode_putp == decode_getp)
	{
		r = EMPTY_OK;
	}
	return r;
}

#ifdef LEVEL
static u1 level2bar(s4 data)
{
	u1 r;

	if(data > (0x8000L/9L * 8L))
	{
		r = 0xff;
	}
	else if(data > (0x8000L/9L * 7L))
	{
		r = 0x7f;
	}
	else if(data > (0x8000L/9L * 6L))
	{
		r = 0x3f;
	}
	else if(data > (0x8000L/9L * 5L))
	{
		r = 0x1f;
	}
	else if(data > (0x8000L/9L * 4L))
	{
		r = 0x0f;
	
	}else if(data > (0x8000L/9L * 3L))
	{
		r = 0x07;
	}
	else if(data > (0x8000L/9L * 2L))
	{
		r = 0x03;
	}
	else if(data > (0x8000L/9L * 1L))
	{
		r = 0x01;
	}
	else
	{
		r = 0;
	}
	return r;
}
#endif /* LEVEL */

void audio(void)
{
	t_stereo decode_data;

	#ifdef LEVEL
	static u4 x44 = 0UL, x10 = 0UL;
	static t_stereo level = {0, 0};
	#ifdef VOLUME
	static t_stereo volume = {0, 0};
	#endif /* VOLUME */
	#endif /* LEVEL */

	if(GET_OK == decode_get(&decode_data))
	{
		#ifdef VOLUME
		decode_data.l = (s2)((decode_data.l * volume.l) / 256);
		decode_data.r = (s2)((decode_data.r * volume.r) / 256);
		#endif /* VOLUME */

		pwm_set_1a((u2)((decode_data.l / 256) + 128));
		pwm_set_1c((u2)((decode_data.r / 256) + 128));

		#ifdef LEVEL
		if(++x44 >= 44UL)
		{
			x44 = 0UL;
			if(level.l < abs((int)decode_data.l))
			{
				level.l = (s2)abs((int)decode_data.l);
			}
			if(level.r < abs((int)decode_data.r))
			{
				level.r = (s2)abs((int)decode_data.r);
			}

			if(++x10 >= 10UL)
			{
				x10 = 0UL;

				portg_l((u1)~level2bar((s4)level.l));
				portg_h((u1)~level2bar((s4)level.r));

				level.l = 0;
				level.r = 0;

				#ifdef VOLUME
				if(ADC_OK == adc_complete())
				{
					volume.l = (s2)(adc_get_ch0() / 256);
					volume.r = (s2)(adc_get_ch1() / 256);
					adc_start();
				}
				#endif /* VOLUME */
			}
		}
		#endif /* LEVEL */

		led_off();
	}
	else
	{
		led_on();
	}
	return;
}
